package picolib;
import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.NoSuchPortException;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * Periodically poll the Pico board via a serial port.
 */
public class PicoBoard implements Runnable
{
    private int resistanceD;
    private int resistanceC;
    private int resistanceB;
    private boolean button;
    private int resistanceA;
    private int light;
    private int sound;
    private int slider;
    private ScheduledExecutorService executor;
    private byte[] lastSample = new byte[18];
    private SerialPort serialPort;

    public PicoBoard(String portName)
            throws BoardIOException
    {
        initialiseSerialPort(portName);
        this.executor = Executors.newScheduledThreadPool(1);
        this.executor.scheduleAtFixedRate(this, 0L, 10L, TimeUnit.MILLISECONDS);
    }

    @Override
    public void run()
    {
        try {
            InputStream in = this.serialPort.getInputStream();
            OutputStream out = this.serialPort.getOutputStream();

            out.write(1);

            Thread.sleep(25); // Seems to be necessary on some (not all) Linux
            // systems to avoid a deadlock. Bug in rxtx library?
            
            byte[] buffer = new byte[18];
            int len = in.read(buffer);
            
            if (len != 18) {
                // System.out.println("Mis-read, length = " + len);
            }
            else {
                // System.out.println("*** Read length correctly!!");
                this.lastSample = buffer;
            }
            decode(this.lastSample);
        } catch (IOException e) {
            System.out.println(e.getLocalizedMessage());
        } catch (InterruptedException ie) {
            
        }
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    public static String[] getPortNames()
    {
        Enumeration ports = CommPortIdentifier.getPortIdentifiers();
        List portNames = new ArrayList();
        while (ports.hasMoreElements()) {
            CommPortIdentifier port = (CommPortIdentifier)ports.nextElement();
            if ((!port.isCurrentlyOwned()) && (port.getPortType() == 1)) {
                portNames.add(port.getName());
            }
        }
        return (String[])portNames.toArray(new String[portNames.size()]);
    }

    private void initialiseSerialPort(String portName) throws BoardIOException
    {
        CommPortIdentifier portIdentifier;
        try {
            portIdentifier = CommPortIdentifier.getPortIdentifier(portName);
        } catch (NoSuchPortException e) {
            throw new BoardIOException(e);
        }
        CommPort commPort;
        try
        {
            commPort = portIdentifier.open(getClass().getName(), 2000);
        } catch (PortInUseException e) {
            throw new BoardIOException(e);
        }

        if ((commPort instanceof SerialPort))
        {
            this.serialPort = ((SerialPort)commPort);
            try {
                this.serialPort.setSerialPortParams(38400, 8, 1, 0);
                this.serialPort.enableReceiveThreshold(18);
            } catch (UnsupportedCommOperationException e) {
                throw new BoardIOException(e);
            }
        }
        else
        {
            throw new BoardIOException(portName + ": Not a serial port");
        }
    }

    private void decode(byte[] buf)
    {
        for (int i = 0; i < 18; i += 2) {
            int chan = (buf[i] & 0x78) >> 3;
            int val = (buf[i] & 0x7) << 7 | buf[(i + 1)] & 0x7F;
            switch (chan) {
            case 0:
                this.resistanceD = generic(val); break;
            case 1:
                this.resistanceC = generic(val); break;
            case 2:
                this.resistanceB = generic(val); break;
            case 3:
                this.button = (val < 512); break;
            case 4:
                this.resistanceA = generic(val); break;
            case 5:
                this.light = light(val); break;
            case 6:
                this.sound = sound(val); break;
            case 7:
                this.slider = generic(val); break;
            case 15:
            case 8:
            case 9:
            case 10:
            case 11:
            case 12:
            case 13:
            case 14:
            }
        } 
    }

    public int getResistanceD() { return this.resistanceD; } 

    public int getResistanceC() { return this.resistanceC; } 

    public int getResistanceB() { return this.resistanceB; } 

    public boolean isButtonPressed() { return this.button; } 

    public int getResistanceA() { return this.resistanceA; } 

    public int getLight() { return this.light; } 

    public int getSound() { return this.sound; } 

    public int getSlider() { return this.slider; }

    public void close()
    {
        this.executor.shutdown();
        this.serialPort.close();
    }

    @Override
    public String toString()
    {
        String res = "Port: " + this.serialPort + "\n";

        res = res + "resistanceA: " + this.resistanceA + "\n";
        res = res + "resistanceB: " + this.resistanceB + "\n";
        res = res + "resistanceC: " + this.resistanceC + "\n";
        res = res + "resistanceD: " + this.resistanceD + "\n";
        res = res + "button: " + this.button + "\n";
        res = res + "light: " + this.light + "\n";
        res = res + "sound: " + this.sound + "\n";
        res = res + "slider: " + this.slider + "\n";

        for (int i = 0; i < 18; i += 2) {
            int chan = (this.lastSample[i] & 0x78) >> 3;
            int val = (this.lastSample[i] & 0x7) << 7 | this.lastSample[(i + 1)] & 0x7F;
            res = res + chan + ": " + val + ", ";
        }
        res = res + "\n";

        return res;
    }

    private int generic(int val)
    {
        return (int)Math.round(100.0D * val / 1023.0D);
    }

    private int light(int val)
    {
        if (val < 25) return 100 - val;
        return (int)Math.round((1023.0D - val) * 0.07515030060120241D);
    }

    private int sound(int val)
    {
        int n = Math.max(0, val - 18);
        if (n < 50) return n / 2;

        return 25 + Math.min(75, (int)Math.round((n - 50.0D) * 0.1293103448275862D));
    }
}
