import java.io.IOException;

import picolib.NativePathSetter;
import picolib.PicoBoard;
import picolib.PortChooserDialog;

/**
 * A Greenfoot version of the PicoBoard class. This version uses the
 * NativeLoader class so the native libraries are only loaded once - 
 * the scenario can therefore be reset without having to restart 
 * Greenfoot.
 * 
 * @author Michael Berry (mjrb4)
 * @version 26/2/12
 */
public class GreenPicoBoard
{
    static {
        NativePathSetter.setPaths();
    }

    private PicoBoard picoBoard;
    
    public GreenPicoBoard()
    {
        this(null);
    }

    /**
     * Create a new pico board on the specified com port.
     */
    public GreenPicoBoard(String port)
    {
        if(port==null) {
            String[] names = PicoBoard.getPortNames();
            if(names.length==0) {
                System.err.println("No boards found.");
            }
            else if(names.length==1) {
                port = names[0];
            }
            else {
                PortChooserDialog dialog = new PortChooserDialog(names);
                dialog.setModal(true);
                dialog.setVisible(true);
                port = dialog.getPort();
            }
        }
        setPicoBoard(port);    
    }
    
    private void setPicoBoard(String port)
    {
        try {
            this.picoBoard = new PicoBoard(port);
        }
        catch(IOException ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Close the picoboard connection. This must be called 
     * otherwise the port won't be freed and Greenfoot will
     * need to be restarted.
     */
    public void close()
    {
        picoBoard.close();
    }
    
    /**
     * Get the level of the sound meter, between 0 and
     * 100.
     */
    public int getSound()
    {
        return picoBoard.getSound();
    }
    
    /**
     * Get the position of the slider, between 0 and
     * 100.
     */
    public int getSlider()
    {
        int n = picoBoard.getSlider();
        return n;
    }
    
    /**
     * Determine whether the button is pressed or not.
     */
    public boolean isButtonPressed()
    {
        return picoBoard.isButtonPressed();
    }
    
    /**
     * Get the value of the light sensor, between 0
     * and 100.
     */
    public int getLight()
    {
        return picoBoard.getLight();
    }
    
    /**
     * Get the resistance of input A (between 0
     * and 100.)
     */
    public int getResistanceA()
    {
        return picoBoard.getResistanceA();
    }
    
    /**
     * Get the resistance of input B (between 0
     * and 100.)
     */
    public int getResistanceB()
    {
        return picoBoard.getResistanceB();
    }
    
    /**
     * Get the resistance of input C (between 0
     * and 100.)
     */
    public int getResistanceC()
    {
        return picoBoard.getResistanceC();
    }
    
    /**
     * Get the resistance of input D (between 0
     * and 100.)
     */
    public int getResistanceD()
    {
        return picoBoard.getResistanceD();
    }
}
